# -*- coding: utf-8 -*-
import sys

from os.path import join, isdir

from django.conf.urls import patterns, url

from djangojs.conf import settings
from djangojs.views import UrlsJsonView, ContextJsonView, JsInitView


def js_info_dict():
    js_info_dict = {
        'packages': [],
    }

    for app in settings.INSTALLED_APPS:
        if settings.JS_I18N_APPS and app not in settings.JS_I18N_APPS:
            continue
        if settings.JS_I18N_APPS_EXCLUDE and app in settings.JS_I18N_APPS_EXCLUDE:
            continue
        module = sys.modules[app]
        for path in module.__path__:
            if isdir(join(path, 'locale')):
                js_info_dict['packages'].append(app)
                break
    return js_info_dict


urlpatterns = patterns('',
    url(r'^init\.js$', JsInitView.as_view(), name='django_js_init'),
    url(r'^urls$', UrlsJsonView.as_view(), name='django_js_urls'),
    url(r'^context$', ContextJsonView.as_view(), name='django_js_context'),
    url(r'^translation$', 'django.views.i18n.javascript_catalog', js_info_dict(), name='js_catalog'),
)



import unittest

from djangojs.tap import TapParser, TapTest, TapModule, TapAssertion


class TapAssertionTest(unittest.TestCase):
    def test_parse_ok(self):
        '''Should parse a simple OK assertion'''
        assertion = TapAssertion.parse('ok 1')
        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 1)
        self.assertEqual(assertion.success, True)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, None)
        self.assertEqual(assertion.expected, None)
        self.assertEqual(assertion.got, None)
        self.assertListEqual(assertion.stack, [])

    def test_parse_not_ok(self):
        '''Should parse a simple NOT OK assertion'''
        assertion = TapAssertion.parse('not ok 2')
        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 2)
        self.assertEqual(assertion.success, False)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, None)
        self.assertEqual(assertion.expected, None)
        self.assertEqual(assertion.got, None)
        self.assertListEqual(assertion.stack, [])

    def test_parse_ok_with_message(self):
        '''Should parse an OK assertion with message'''
        assertion = TapAssertion.parse('ok 284 - I should be equal to me.')
        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 284)
        self.assertEqual(assertion.success, True)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, 'I should be equal to me.')
        self.assertEqual(assertion.expected, None)
        self.assertEqual(assertion.got, None)
        self.assertListEqual(assertion.stack, [])

    def test_parse_not_ok_with_message(self):
        '''Should parse a NOT OK assertion with message'''
        assertion = TapAssertion.parse('not ok 284 - I should be equal to me.')
        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 284)
        self.assertEqual(assertion.success, False)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, 'I should be equal to me.')
        self.assertEqual(assertion.expected, None)
        self.assertEqual(assertion.got, None)
        self.assertListEqual(assertion.stack, [])

    def test_parse_not_ok_with_source(self):
        '''Should parse a NOT OK assertion with message and source'''
        line = 'not ok 298 - reset should not modify test status, source: at http://localhost:8000/static/js/test/libs/qunit.js:435'

        assertion = TapAssertion.parse(line)

        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 298)
        self.assertEqual(assertion.success, False)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, 'reset should not modify test status')
        self.assertEqual(assertion.expected, None)
        self.assertEqual(assertion.got, None)
        self.assertListEqual(assertion.stack, ['http://localhost:8000/static/js/test/libs/qunit.js:435'])

    def test_parse_not_ok_with_expectations(self):
        '''Should parse a NOT OK assertion with expectations'''
        line = "not ok 42 - expected: 'something', got: 'something else'"

        assertion = TapAssertion.parse(line)

        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 42)
        self.assertEqual(assertion.success, False)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, None)
        self.assertEqual(assertion.expected, 'something')
        self.assertEqual(assertion.got, 'something else')
        self.assertListEqual(assertion.stack, [])

    def test_parse_not_ok_with_all(self):
        '''Should parse a NOT OK assertion with all extras'''
        line = "not ok 42 - reset should not modify test status, expected: 'something', got: 'something else', source: at http://localhost:8000/static/js/test/libs/qunit.js:435"

        assertion = TapAssertion.parse(line)

        self.assertIsNotNone(assertion)
        self.assertEqual(assertion.num, 42)
        self.assertEqual(assertion.success, False)
        self.assertEqual(assertion.parsed_indent, '')
        self.assertEqual(assertion.message, 'reset should not modify test status')
        self.assertEqual(assertion.expected, 'something')
        self.assertEqual(assertion.got, 'something else')
        self.assertListEqual(assertion.stack, ['http://localhost:8000/static/js/test/libs/qunit.js:435'])


class TapTestTest(unittest.TestCase):
    def test_parse_test(self):
        '''Should parse a test statement'''
        line = '# test: This is a test'

        test = TapTest.parse(line)

        self.assertEqual(test.name, 'This is a test')
        self.assertEqual(test.parsed_indent, '')


class TapModuleTest(unittest.TestCase):
    def test_parse_module(self):
        '''Should parse a module statement'''
        line = '# module: This is a module'

        module = TapModule.parse(line)

        self.assertEqual(module.name, 'This is a module')
        self.assertEqual(module.parsed_indent, '')


class TapParserTest(unittest.TestCase):
    def test_single_test(self):
        '''Should parse a test and its children'''
        # import ipdb; ipdb.set_trace()
        parser = TapParser(TapAssertion)
        output = '''
# test: should be defined
ok 1
not ok 2
        '''

        items = list(parser.parse(output.splitlines()))

        self.assertIsInstance(items[0], TapTest)
        self.assertIsInstance(items[1], TapAssertion)
        self.assertIsInstance(items[2], TapAssertion)

    def test_single_module(self):
        '''Should parse a module and its children'''
        parser = TapParser(TapAssertion)
        output = '''
# module: This is a module
# test: should be defined
ok 1
        '''

        items = list(parser.parse(output.splitlines()))

        self.assertIsInstance(items[0], TapModule)
        self.assertIsInstance(items[1], TapTest)
        self.assertIsInstance(items[2], TapAssertion)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

from django.conf import global_settings
from django.contrib.auth.management import create_permissions
from django.contrib.auth.models import User
from django.contrib.contenttypes.management import update_contenttypes
from django.core.urlresolvers import reverse
from django.db.models import get_app
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.utils import translation
from django.middleware.locale import LocaleMiddleware
from django.contrib.sessions.middleware import SessionMiddleware

from djangojs.conf import settings
from djangojs.utils import ContextSerializer

TEST_CONTEXT_PROCESSORS = global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
    'djangojs.tests.custom_processor',
    'django.core.context_processors.i18n',
)

TEST_MIDDLEWARES = global_settings.MIDDLEWARE_CLASSES + (
    'django.middleware.locale.LocaleMiddleware',
)


class ContextTestMixin(object):

    def setUp(self):
        self.factory = RequestFactory()

    def process_request(self, admin=False, headers=None):
        headers = headers if headers else {}
        request = self.factory.get(reverse('django_js_context'), **headers)
        SessionMiddleware().process_request(request)
        LocaleMiddleware().process_request(request)
        if admin:
            request.user = User.objects.create_superuser('admin', 'fake@noirbizarre.info', 'password')
        else:
            request.user = User.objects.create_user('user', 'fake@noirbizarre.info', 'password')
        return self.get_result(request)

    def test_static_url(self):
        '''STATIC_URL should be in context'''
        result = self.process_request()
        self.assertIn('STATIC_URL', result)
        self.assertEqual(result['STATIC_URL'], settings.STATIC_URL)

    def test_media_url(self):
        '''MEDIA_URL should be in context'''
        result = self.process_request()
        self.assertIn('MEDIA_URL', result)
        self.assertEqual(result['MEDIA_URL'], settings.MEDIA_URL)

    @override_settings(LANGUAGE_CODE='en-us')
    def test_language_code(self):
        '''LANGUAGE_CODE should be in context'''
        result = self.process_request(headers={'HTTP_ACCEPT_LANGUAGE': 'fr'})
        self.assertIn('LANGUAGE_CODE', result)
        self.assertEqual(result['LANGUAGE_CODE'], 'fr')

    @override_settings(LANGUAGE_CODE='en-us')
    def test_language_code_default(self):
        '''Should handle the default LANGUAGE_CODE="en-us"'''
        result = self.process_request()
        self.assertIn('LANGUAGE_CODE', result)
        self.assertEqual(result['LANGUAGE_CODE'], 'en-us')

    def test_language_bidi(self):
        '''LANGUAGE_BIDI should be in context'''
        result = self.process_request()
        self.assertIn('LANGUAGE_BIDI', result)
        self.assertEqual(result['LANGUAGE_BIDI'], translation.get_language_bidi())

    @override_settings(LANGUAGE_CODE='en-us')
    def test_language_name(self):
        '''LANGUAGE_NAME should be in context'''
        result = self.process_request(headers={'HTTP_ACCEPT_LANGUAGE': 'fr'})
        self.assertIn('LANGUAGE_NAME', result)
        self.assertEqual(result['LANGUAGE_NAME'], 'French')

    @override_settings(LANGUAGE_CODE='en-us')
    def test_language_name_en_us(self):
        '''LANGUAGE_NAME should handle en-us special case'''
        result = self.process_request()
        self.assertIn('LANGUAGE_NAME', result)
        # code = translation.get_language()
        # code = 'en' if code == 'en-us' else code
        # language = translation.get_language_info(code)
        self.assertEqual(result['LANGUAGE_NAME'], 'English')

    def test_language_name_local(self):
        '''LANGUAGE_NAME_LOCAL should be in context'''
        result = self.process_request()
        self.assertIn('LANGUAGE_NAME_LOCAL', result)
        code = translation.get_language()
        code = 'en' if code == 'en-us' else code
        language = translation.get_language_info(code)
        self.assertEqual(result['LANGUAGE_NAME_LOCAL'], language['name_local'])

    @override_settings(DEBUG=True, LANGUAGE_CODE=global_settings.LANGUAGE_CODE)
    def test_language_debug(self):
        '''LANGUAGE_* should be present even in debug with default language'''
        result = self.process_request()
        self.assertIn('LANGUAGE_BIDI', result)
        self.assertIn('LANGUAGE_CODE', result)
        self.assertIn('LANGUAGE_NAME', result)
        self.assertIn('LANGUAGE_NAME_LOCAL', result)

    @override_settings(LANGUAGE_CODE='en-us')
    def test_languages(self):
        '''LANGUAGES should be in context'''
        result = self.process_request()
        self.assertIn('LANGUAGES', result)
        languages = result['LANGUAGES']
        self.assertTrue(isinstance(languages, dict))
        for code, name in settings.LANGUAGES:
            self.assertEqual(languages[code], name)

    @override_settings(LANGUAGE_CODE='en-us')
    def test_languages_localized(self):
        '''LANGUAGES should be localized'''
        result = self.process_request(headers={'HTTP_ACCEPT_LANGUAGE': 'fr'})
        self.assertIn('LANGUAGES', result)
        languages = result['LANGUAGES']
        self.assertTrue(isinstance(languages, dict))
        translation.activate('fr')
        for code, name in settings.LANGUAGES:
            self.assertEqual(languages[code], translation.ugettext_lazy(name))

    def test_any_custom_context_processor(self):
        '''Any custom context processor should be in context'''
        result = self.process_request()
        self.assertIn('CUSTOM', result)
        self.assertEqual(result['CUSTOM'], 'CUSTOM_VALUE')

    def test_user(self):
        '''Basic user informations should be in context'''
        result = self.process_request()
        self.assertIn('user', result)
        self.assertIn('username', result['user'])
        self.assertEqual(result['user']['username'], 'user')
        self.assertIn('is_authenticated', result['user'])
        self.assertTrue(result['user']['is_authenticated'])
        self.assertIn('is_staff', result['user'])
        self.assertFalse(result['user']['is_staff'])
        self.assertIn('is_superuser', result['user'])
        self.assertFalse(result['user']['is_superuser'])
        self.assertIn('permissions', result['user'])
        self.assertTrue(isinstance(result['user']['permissions'], (list, tuple)))

    def test_super_user(self):
        '''Basic superuser informations should be in context'''
        result = self.process_request(True)

        self.assertEqual(result['user']['username'], 'admin')
        self.assertTrue(result['user']['is_authenticated'])
        self.assertTrue(result['user']['is_staff'])
        self.assertTrue(result['user']['is_superuser'])

    def test_user_permissions(self):
        '''Should list permissions'''
        self.fake_permissions()
        result = self.process_request(True)

        self.assertIn('permissions', result['user'])
        self.assertTrue(isinstance(result['user']['permissions'], (list, tuple)))

        # Default permissions
        for perm in ('add', 'change', 'delete'):
            self.assertIn('fake.%s_fakemodel' % perm, result['user']['permissions'])
        # Custom permissions
        for perm in ('do_something', 'do_something_else'):
            self.assertIn('fake.%s' % perm, result['user']['permissions'])

    def test_user_without_permissions(self):
        '''Should not list denied permissions'''
        result = self.process_request()
        self.assertIn('permissions', result['user'])
        self.assertTrue(isinstance(result['user']['permissions'], (list, tuple)))
        self.assertEqual(len(result['user']['permissions']), 0)

    def fake_permissions(self):
        ''' Add fake app missing content types and permissions'''
        app = get_app('fake')
        update_contenttypes(app, None, 0)
        create_permissions(app, None, 0)


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=TEST_CONTEXT_PROCESSORS,
    MIDDLEWARE_CLASSES=TEST_MIDDLEWARES,
    INSTALLED_APPS=['djangojs', 'djangojs.fake']
)
class ContextAsDictTest(ContextTestMixin, TestCase):

    def get_result(self, request):
        return ContextSerializer.as_dict(request)


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=TEST_CONTEXT_PROCESSORS,
    MIDDLEWARE_CLASSES=TEST_MIDDLEWARES,
    INSTALLED_APPS=['djangojs', 'djangojs.fake']
)
class ContextAsJsonTest(ContextTestMixin, TestCase):

    def get_result(self, request):
        return json.loads(ContextSerializer.as_json(request))


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=TEST_CONTEXT_PROCESSORS,
    MIDDLEWARE_CLASSES=TEST_MIDDLEWARES,
    INSTALLED_APPS=['djangojs', 'djangojs.fake']
)
class ContextJsonViewTest(TestCase):
    urls = 'djangojs.test_urls'

    def test_render(self):
        '''It should render a JSON Context descriptor'''
        response = self.client.get(reverse('django_js_context'))

        self.assertIsNotNone(response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')
        self.assertIsNotNone(json.loads(response.content.decode()))



import json

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import six


class UrlsTestMixin(object):
    urls = 'djangojs.test_urls'

    def setUp(self):
        self.result = self.get_result()

    def test_simple_url(self):
        '''It should serialize a simple URL without parameters'''
        self.assertIn('django_js_urls', self.result)
        self.assertEqual(self.result['django_js_urls'], '/djangojs/urls')

    def test_url_an_arg(self):
        '''It should serialize an URL with a single anonymous parameter'''
        self.assertIn('test_arg', self.result)
        self.assertEqual(self.result['test_arg'], '/test/arg/<>')

    def test_url_many_args(self):
        '''It should serialize an URL with many anonymous parameters'''
        self.assertIn('test_arg_multi', self.result)
        self.assertEqual(self.result['test_arg_multi'], '/test/arg/<>/<>')

    def test_url_a_kwarg(self):
        '''It should serialize an URL with a single named parameter'''
        self.assertIn('test_named', self.result)
        self.assertEqual(self.result['test_named'], '/test/named/<test>')

    def test_url_many_kwargs(self):
        '''It should serialize an URL with many named parameters'''
        self.assertIn('test_named_multi', self.result)
        self.assertEqual(self.result['test_named_multi'], '/test/named/<str>/<num>')

    def test_unnamed_url(self):
        '''It should not serialize unnamed URLs by default'''
        self.assertNotIn('', self.result)
        for value in six.itervalues(self.result):
            self.assertNotEqual(value, '/test/unnamed')
            self.assertNotEqual(value, '/test/unnamed-class')

    @override_settings(JS_URLS_UNNAMED=True)
    def test_unnamed_url_set(self):
        '''It should serialize unnamed url if JS_URLS_UNNAMED is set'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertIn('djangojs.test_urls.unnamed', self.result)

    @override_settings(JS_URLS_UNNAMED=True)
    def test_unnamed_url_set_class(self):
        '''It should not serialize unnamed url for class based views if JS_URLS_UNNAMED is set'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertNotIn('djangojs.test_urls.TestFormView', self.result)

    def test_optionnal_chars(self):
        '''It should not serialize optionnal characters (take the shortest)'''
        self.assertIn('opt', self.result)
        url = self.result['opt']
        self.assertNotIn('?', url)
        self.assertEqual(url, '/test/optionnal')

        self.assertIn('opt_multi', self.result)
        url = self.result['opt_multi']
        self.assertNotIn('?', url)
        self.assertEqual(url, '/test/man/optionnal')

    def test_optionnal_groups(self):
        '''It should not serialize optionnal non capturing groups'''
        self.assertIn('opt_grp', self.result)
        url = self.result['opt_grp']
        self.assertNotIn('?', url)
        self.assertEqual(url, '/test/optionnal/group')

    def test_keep_priority(self):
        '''It should serialize with priority support'''
        self.assertIn('twice', self.result)
        self.assertEqual(self.result['twice'], reverse('twice'))

    def test_escape(self):
        '''It should unescape escaped characters'''
        self.assertIn('escaped', self.result)
        self.assertEqual(self.result['escaped'], reverse('escaped'))

    def test_single_namespace(self):
        '''It should serialize namespaces'''
        self.assertIn('ns1:fake', self.result)
        self.assertEqual(self.result['ns1:fake'], reverse('ns1:fake'))
        self.assertNotIn('fake', self.result)

    def test_nested_namespaces(self):
        '''It should serialize nested namespaces'''
        self.assertIn('ns2:nested:fake', self.result)
        self.assertEqual(self.result['ns2:nested:fake'], reverse('ns2:nested:fake'))
        self.assertNotIn('nested:fake', self.result)
        self.assertNotIn('ns2:nested', self.result)
        self.assertNotIn('ns2:fake', self.result)
        self.assertNotIn('fake', self.result)
        self.assertNotIn('nested', self.result)

    def test_single_instance_namespace(self):
        '''It should serialize instance namespaces'''
        self.assertIn('app1:fake', self.result)
        self.assertEqual(self.result['app1:fake'], reverse('app1:fake'))

    def test_nested_instance_namespaces(self):
        '''It should serialize nested instance namespaces'''
        self.assertIn('ns2:appnested:fake', self.result)
        self.assertIn('app2:nested:fake', self.result)
        self.assertIn('app2:appnested:fake', self.result)
        self.assertEqual(self.result['ns2:appnested:fake'], reverse('ns2:appnested:fake'))
        self.assertEqual(self.result['app2:nested:fake'], reverse('app2:nested:fake'))
        self.assertEqual(self.result['app2:appnested:fake'], reverse('app2:appnested:fake'))
        self.assertNotIn('appnested:fake', self.result)
        self.assertNotIn('ns2:appnested', self.result)
        self.assertNotIn('app2:fake', self.result)
        self.assertNotIn('fake', self.result)

    def test_single_namespace_without_app_name(self):
        '''It should serialize namespaces without app_name set'''
        self.assertIn('ns3:fake', self.result)
        self.assertEqual(self.result['ns3:fake'], reverse('ns3:fake'))
        self.assertNotIn('fake', self.result)

    @override_settings(JS_URLS=['django_js_urls'])
    def test_urls_whitelist(self):
        '''Should only include urls listed in JS_URLS'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertIn('django_js_urls', self.result)
        self.assertNotIn('test_arg', self.result)

    @override_settings(JS_URLS_EXCLUDE=['django_js_urls'])
    def test_urls_blacklist(self):
        '''Should exclude urls listed in JS_URLS_EXCLUDE'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertNotIn('django_js_urls', self.result)
        self.assertIn('test_arg', self.result)

    @override_settings(JS_URLS_NAMESPACES=['ns1'])
    def test_urls_namespaces_whitelist(self):
        '''Should only include namespaces listed in JS_URLS_NAMESPACES'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertIn('ns1:fake', self.result)
        self.assertNotIn('ns2:nested:fake', self.result)

    @override_settings(JS_URLS_NAMESPACES_EXCLUDE=['ns1'])
    def test_urls_namespaces_blacklist(self):
        '''Should exclude namespaces listed in JS_URLS_NAMESPACES_EXCLUDE'''
        self.result = self.get_result()  # To take override_settings in account
        self.assertNotIn('ns1:fake', self.result)
        self.assertIn('ns2:nested:fake', self.result)


class UrlsAsDictTest(UrlsTestMixin, TestCase):

    def get_result(self):
        from djangojs.utils import urls_as_dict
        return urls_as_dict()


class UrlsAsJsonTest(UrlsTestMixin, TestCase):

    def get_result(self):
        from djangojs.utils import urls_as_json
        return json.loads(urls_as_json())


class UrlsJsonViewTest(UrlsTestMixin, TestCase):

    def get_result(self):
        self.response = self.client.get(reverse('django_js_urls'))
        return json.loads(self.response.content.decode())

    def test_render(self):
        '''It should render a JSON URLs descriptor'''
        self.assertIsNotNone(self.response)
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response['Content-Type'], 'application/json')
        self.assertIsNotNone(self.result)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sys

from django.conf import settings as _settings

from djangojs import JQUERY_DEFAULT_VERSION

# Default configuration values for Django.js
# All values used by Django.js needs to appears here.
DEFAULTS = {
    'DEBUG': False,
    'TESTING': 'test' in sys.argv,
    'JS_URLS': None,
    'JS_URLS_EXCLUDE': None,
    'JS_URLS_NAMESPACES': None,
    'JS_URLS_NAMESPACES_EXCLUDE': None,
    'JS_URLS_UNNAMED': False,
    'JS_I18N_APPS': None,
    'JS_I18N_APPS_EXCLUDE': None,
    'JQUERY_VERSION': JQUERY_DEFAULT_VERSION,
}


class DjangoJsSettings(object):
    '''
    Lazy Django settings wrapper for Django.js
    '''
    def __init__(self, wrapped_settings):
        self.wrapped_settings = wrapped_settings

    def __getattr__(self, name):
        if hasattr(self.wrapped_settings, name):
            return getattr(self.wrapped_settings, name)
        elif name in DEFAULTS:
            return DEFAULTS[name]
        else:
            raise AttributeError("'%s' setting not found" % name)

settings = DjangoJsSettings(_settings)



# -*- coding: utf-8 -*-
from django.db import models


class FakeModel(models.Model):
    something = models.CharField(max_length=256)

    class Meta:
        permissions = (
            ("do_something", "Can do something"),
            ("do_something_else", "Can do something else"),
        )



# -*- coding: utf-8 -*-
'''
Django.js provide better integration of javascript into Django.
'''

__version__ = '0.7.4'
__description__ = "Django JS Tools"

#: Packaged jQuery version
JQUERY_DEFAULT_VERSION = '2.0.0'
JQUERY_MIGRATE_VERSION = '1.1.1'



from django.test import TestCase

from djangojs.utils import StorageGlobber


class StorageGlobberTest(TestCase):

    def test_no_js_file(self):
        '''Should handle empty js file list'''
        # view = JsTestView()
        self.assertEqual(StorageGlobber.glob(), [])

    def test_single_js_file(self):
        '''Should handle a single js file name as string'''
        files = 'js/test/libs/jasmine-djangojs.js'
        expected = ['js/test/libs/jasmine-djangojs.js']
        self.assertEqual(StorageGlobber.glob(files), expected)

    def test_multi_js_file(self):
        '''Should handle an array of js file names'''
        files = ['js/test/libs/jasmine-djangojs.js', 'js/test/libs/jasmine.js']
        result = StorageGlobber.glob(files)

        self.assertIn('js/test/libs/jasmine-djangojs.js', result)
        self.assertIn('js/test/libs/jasmine.js', result)

    def test_single_glob_expression(self):
        '''Should handle a single glob pattern as js file list'''
        files = 'js/test/libs/jasmine-*.js'
        result = StorageGlobber.glob(files)

        self.assertIn('js/test/libs/jasmine-djangojs.js', result)
        self.assertIn('js/test/libs/jasmine-html.js', result)
        self.assertIn('js/test/libs/jasmine-jquery.js', result)
        self.assertNotIn('js/test/libs/jasmine.js', result)

    def test_multi_glob_expression(self):
        '''Should handle a glob pattern list as js file list'''
        files = ['js/test/libs/jasmine-*.js', 'js/test/libs/qunit-*.js']
        result = StorageGlobber.glob(files)

        self.assertIn('js/test/libs/jasmine-djangojs.js', result)
        self.assertIn('js/test/libs/jasmine-html.js', result)
        self.assertIn('js/test/libs/jasmine-jquery.js', result)
        self.assertIn('js/test/libs/qunit-tap.js', result)
        self.assertNotIn('js/test/libs/jasmine.js', result)
        self.assertNotIn('js/test/libs/qunit.js', result)

    def test_preserve_order(self):
        '''Should preserve declaration order'''
        # Orders matters: should not be an alphabeticaly sorted list
        files = ['js/test/libs/jasmine.js', 'js/djangojs/django.js', 'js/test/libs/qunit-*.js']
        result = StorageGlobber.glob(files)

        self.assertEqual(result[0], 'js/test/libs/jasmine.js')
        self.assertEqual(result[1], 'js/djangojs/django.js')

        for lib in result[2:]:
            self.assertIn('js/test/libs/qunit-', lib)
            self.assertTrue(lib.endswith('.js'))



# -*- coding: utf-8 -*-
'''
This module provide Javascript test runners for Django unittest.
'''
from __future__ import unicode_literals

import os
import re
import sys

from os.path import join, dirname
from subprocess import Popen, STDOUT, PIPE
from tempfile import NamedTemporaryFile, mkstemp

from django.contrib.staticfiles import finders
from django.core.files.storage import FileSystemStorage
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.test import LiveServerTestCase
from django.utils.encoding import python_2_unicode_compatible

from djangojs.tap import TapParser
from djangojs.utils import StorageGlobber

from unittest import TestCase

#: Console output line length for separators
LINE_SIZE = 70

__all__ = (
    'JsTestCase',
    'JsFileTestCase',
    'JsTemplateTestCase',
    'JsTestException',
    'JasmineSuite',
    'QUnitSuite',
    'AbsoluteFileStorage',
)


def parse_verbosity():
    verbosity = 1
    regex = re.compile(r'(?:-v|--verbosity)(?:(?:\=)?([0-3]))?')
    for idx, arg in enumerate(sys.argv):
        match = regex.match(arg)
        if match:
            verbosity = int(match.group(1)) if match.group(1) else int(sys.argv[idx + 1])
            break
    return verbosity

VERBOSITY = parse_verbosity()
VERBOSE = VERBOSITY > 1


@python_2_unicode_compatible
class JsTestException(Exception):
    '''
    An exception raised by Javascript tests.

    It display javascript errors into the exception message.
    '''
    def __init__(self, message, failures=None):
        super(JsTestException, self).__init__(message)
        self.failures = failures or []

    def __str__(self):
        output = [super(JsTestException, self).__str__()]
        for failure in self.failures:
            message = failure.message or "expected: '%(expected)s', got: '%(got)s'" % failure.__dict__
            output.append(' + %s - %s' % (failure.parent.name, message))
            for line in failure.stack:
                # Remove jasmine/qunit entries in stack to produce smaller and more efficient output
                # TODO: find a cleaner way to improve stack trace output
                if '/js/test/libs/jasmine.js:' in line or '/js/test/libs/qunit.js:' in line:
                    continue
                output.append('\t%s' % line)
        return '\n'.join(output)


class PhantomJsRunner(object):
    '''
    Test helper to run JS tests with PhantomJS
    '''
    #: mandatory path to the PhantomJS javascript runner
    phantomjs_runner = None
    #: an optionnal absolute URL to the test runner page
    url = None
    #: an optionnal title for verbose console output
    title = 'PhantomJS test suite'
    #: PhantomJS execution timeout in seconds
    timeout = 3

    def execute(self, command):
        '''
        Execute a subprocess yielding output lines
        '''
        process = Popen(command, stdout=PIPE, stderr=STDOUT, universal_newlines=True)
        while True:
            if process.poll() is not None:
                self.returncode = process.returncode  # pylint: disable=W0201
                break
            yield process.stdout.readline()

    def phantomjs(self, *args, **kwargs):
        '''
        Execute PhantomJS by giving ``args`` as command line arguments.

        If test are run in verbose mode (``-v/--verbosity`` = 2), it output:
          - the title as header (with separators before and after)
          - modules and test names
          - assertions results (with ``django.utils.termcolors`` support)

        In case of error, a JsTestException is raised to give details about javascript errors.
        '''
        separator = '=' * LINE_SIZE
        title = kwargs['title'] if 'title' in kwargs else 'phantomjs output'
        nb_spaces = (LINE_SIZE - len(title)) // 2

        if VERBOSE:
            print('')
            print(separator)
            print(' ' * nb_spaces + title)
            print(separator)
            sys.stdout.flush()

        with NamedTemporaryFile(delete=True) as cookies_file:
            cmd = ('phantomjs', '--cookies-file=%s' % cookies_file.name) + args
            if self.timeout:
                cmd += (str(self.timeout),)
            parser = TapParser(debug=VERBOSITY > 2)
            output = self.execute(cmd)

            for item in parser.parse(output):
                if VERBOSE:
                    print(item.display())
                    sys.stdout.flush()

        if VERBOSE:
            print(separator)
            sys.stdout.flush()

        failures = parser.suites.get_all_failures()
        if failures:
            raise JsTestException('Failed javascript assertions', failures)
        if self.returncode > 0:
            raise JsTestException('PhantomJS return with non-zero exit code (%s)' % self.returncode)

    def run_suite(self):
        '''
        Run a phantomjs test suite.

         - ``phantomjs_runner`` is mandatory.
         - Either ``url`` or ``url_name`` needs to be defined.
        '''
        if not self.phantomjs_runner:
            raise JsTestException('phantomjs_runner need to be defined')

        url = self.get_url()

        self.phantomjs(self.phantomjs_runner, url, title=self.title)
        self.cleanup()

    def get_url(self):
        if not self.url:
            raise JsTestException('url need to be defined')
        return self.url

    def cleanup(self):
        pass


class JsTestCase(PhantomJsRunner, LiveServerTestCase):
    '''
    A PhantomJS suite that run against the Django LiveServerTestCase
    '''
    #: a mandatory named URL that point to the test runner page
    url_name = None
    #: an optionnal arguments array to pass to the ``reverse()`` function
    url_args = None
    #: an optionnal keyword arguments dictionnary to pass to the ``reverse()`` function
    url_kwargs = None

    def get_url(self):
        if not self.url_name:
            raise JsTestException('url_name need to be defined')

        reversed_url = reverse(self.url_name, args=self.url_args, kwargs=self.url_kwargs)
        return ''.join([self.live_server_url, reversed_url])


class JsFileTestCase(PhantomJsRunner, TestCase):
    '''
    A PhantomJS suite that run against a local html file
    '''
    #: absolute path to the test runner page
    filename = None

    def get_url(self):
        if not self.filename:
            raise JsTestException('filename need to be defined')
        return 'file://%s' % self.filename


class AbsoluteFileStorage(FileSystemStorage):
    '''
    A storage that give the absolute file scheme URL as URL.
    '''
    def url(self, name):
        return 'file://%s' % finders.find(name)


class JsTemplateTestCase(JsFileTestCase):
    '''
    A PhantomJS suite that run against a rendered html file but without server.

    .. note::

        Template is rendered using a modified static storage that give ``file://`` scheme URLs.
        To benefits from it, you have to use either the ``static`` template tag
        or one the djangojs template tags.


    .. warning::

        Template is not rendered within a request/response dialog.
        You can't access the request object and everything that depends on the server.
    '''
    #: absolute path to the test runner page
    template_name = None
    #: A path or a list of path to javascript files to include into the view.
    #:
    #: - Supports glob patterns.
    #: - Order is kept for rendering.
    js_files = None
    #: Includes or not jQuery in the test view.
    #: Template has to handle the ``use_jquery`` property.
    jquery = False

    def setUp(self):
        from django.contrib.staticfiles.storage import staticfiles_storage
        self.storage_bak = staticfiles_storage._wrapped
        staticfiles_storage._wrapped = AbsoluteFileStorage()

    def tearDown(self):
        from django.contrib.staticfiles.storage import staticfiles_storage
        staticfiles_storage._wrapped = self.storage_bak

    def get_url(self):
        if not self.template_name:
            raise JsTestException('template_name need to be defined')

        fd, self.filename = mkstemp(suffix='.html')
        os.fdopen(fd, 'w').write(render_to_string(self.template_name, self.get_context_data()))
        return super(JsTemplateTestCase, self).get_url()

    def get_context_data(self, **kwargs):
        return {
            'js_test_files': StorageGlobber.glob(self.js_files),
            'use_query': self.jquery
        }

    def cleanup(self):
        os.remove(self.filename)


class JasmineSuite(object):
    '''
    A mixin that runs a jasmine test suite with PhantomJs.
    '''
    title = 'Jasmine test suite'
    phantomjs_runner = join(dirname(__file__), 'phantomjs', 'jasmine-runner.js')
    template_name = 'djangojs/jasmine-runner.html'

    def test(self):
        self.run_suite()


class QUnitSuite(object):
    '''
    A mixin that runs a QUnit test suite with PhantomJs.
    '''
    title = 'QUnit test suite'
    phantomjs_runner = join(dirname(__file__), 'phantomjs', 'qunit-runner.js')
    template_name = 'djangojs/qunit-runner.html'

    def test(self):
        self.run_suite()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


def booleans(request):
    '''
    Allow to use booleans in templates.

    See: http://stackoverflow.com/questions/4557114/django-custom-template-tag-which-accepts-a-boolean-parameter
    '''
    return {
        'True': True,
        'true': True,
        'False': False,
        'false': False,
    }



from django.test.utils import override_settings
from django.conf import global_settings

from djangojs.runners import JsTestCase, JsTemplateTestCase, JasmineSuite, QUnitSuite


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'djangojs.tests.custom_processor',
    )
)
class DjangoJsTests(JasmineSuite, JsTestCase):
    urls = 'djangojs.test_urls'
    url_name = 'djangojs_tests'


class JasmineTests(JasmineSuite, JsTestCase):
    urls = 'djangojs.test_urls'
    url_name = 'djangojs_jasmine_tests'


class JasmineTemplateTests(JasmineSuite, JsTemplateTestCase):
    js_files = 'js/test/jasmine/*Spec.js'


class QUnitTests(QUnitSuite, JsTestCase):
    urls = 'djangojs.test_urls'
    url_name = 'djangojs_qunit_tests'


class QUnitTemplateTests(QUnitSuite, JsTemplateTestCase):
    template_name = 'djangojs/test/qunit-test-runner.html'
    js_files = 'js/test/qunit/qunit-*.js'



# -*- coding: utf-8 -*-
'''
This modules holds every helpers that does not fit in any standard django modules.

It might be splitted in futur releases.
'''
from __future__ import unicode_literals

import json
import logging
import os
import re
import sys
import types

from django.contrib.staticfiles import finders
from django.contrib.staticfiles.utils import matches_patterns
from django.core.serializers.json import DjangoJSONEncoder
from django.core.urlresolvers import RegexURLPattern, RegexURLResolver
from django.template.context import RequestContext
from django.utils import translation, six
from django.utils.encoding import force_text
from django.utils.functional import Promise
from django.utils.translation import ugettext_lazy as _

from djangojs.conf import settings

logger = logging.getLogger(__name__)


__all__ = (
    'urls_as_dict',
    'urls_as_json',
    'ContextSerializer',
    'StorageGlobber',
)

RE_KWARG = re.compile(r"(\(\?P\<(.*?)\>.*?\))")  # Pattern for recongnizing named parameters in urls
RE_ARG = re.compile(r"(\(.*?\))")  # Pattern for recognizing unnamed url parameters
RE_OPT = re.compile(r"\w\?")  # Pattern for recognizing optionnal character
RE_OPT_GRP = re.compile(r"\(\?\:.*\)\?")  # Pattern for recognizing optionnal group
RE_ESCAPE = re.compile(r'([^\\]?)\\')  # Recognize escape characters
RE_START_END = re.compile(r'[\$\^]')  # Recognize start and end charaters


def urls_as_dict():
    '''
    Get the URLs mapping as a dictionnary
    '''
    module = settings.ROOT_URLCONF
    return _get_urls(module)


def urls_as_json():
    '''
    Get the URLs mapping as JSON
    '''
    return json.dumps(urls_as_dict(), cls=DjangoJSONEncoder)


def _get_urls(module, prefix='', namespace=None):
    urls = {}
    if isinstance(module, (six.text_type, six.string_types)):
        __import__(module)
        root_urls = sys.modules[module]
        patterns = root_urls.urlpatterns
    elif isinstance(module, (list, tuple)):
        patterns = module
    elif isinstance(module, types.ModuleType):
        patterns = module.urlpatterns
    else:
        raise TypeError('Unsupported type: %s' % type(module))

    for pattern in patterns:
        if issubclass(pattern.__class__, RegexURLPattern):
            if settings.JS_URLS_UNNAMED:
                mod_name, obj_name = pattern.callback.__module__, pattern.callback.__name__
                try:
                    module = __import__(mod_name, fromlist=[obj_name])
                    obj = getattr(module, obj_name)
                    func_name = "{0}.{1}".format(mod_name, obj_name) if isinstance(obj, types.FunctionType) else None
                    pattern_name = pattern.name or func_name
                except:
                    pattern_name = pattern.name
            else:
                pattern_name = pattern.name

            if pattern_name:
                if settings.JS_URLS and pattern_name not in settings.JS_URLS:
                    continue
                if settings.JS_URLS_EXCLUDE and pattern_name in settings.JS_URLS_EXCLUDE:
                    continue
                if namespace:
                    pattern_name = ':'.join((namespace, pattern_name))
                full_url = prefix + pattern.regex.pattern
                for char in ['^', '$']:
                    full_url = full_url.replace(char, '')
                # remove optionnal non capturing groups
                opt_grp_matches = RE_OPT_GRP.findall(full_url)
                if opt_grp_matches:
                    for match in opt_grp_matches:
                        full_url = full_url.replace(match, '')
                # remove optionnal characters
                opt_matches = RE_OPT.findall(full_url)
                if opt_matches:
                    for match in opt_matches:
                        full_url = full_url.replace(match, '')
                # handle kwargs, args
                kwarg_matches = RE_KWARG.findall(full_url)
                if kwarg_matches:
                    for el in kwarg_matches:
                        # prepare the output for JS resolver
                        full_url = full_url.replace(el[0], "<%s>" % el[1])
                # after processing all kwargs try args
                args_matches = RE_ARG.findall(full_url)
                if args_matches:
                    for el in args_matches:
                        full_url = full_url.replace(el, "<>")  # replace by a empty parameter name
                # Unescape charaters
                full_url = RE_ESCAPE.sub(r'\1', full_url)

                urls[pattern_name] = "/" + full_url
        elif issubclass(pattern.__class__, RegexURLResolver):
            if pattern.urlconf_name:
                if pattern.namespace and not pattern.app_name:
                    # Namespace without app_name
                    nss = [pattern.namespace]
                else:
                    # Add urls twice: for app and instance namespace
                    nss = set((pattern.namespace, pattern.app_name))
                for ns in nss:
                    namespaces = [nsp for nsp in (namespace, ns) if nsp]
                    namespaces = ':'.join(namespaces)
                    if settings.JS_URLS_NAMESPACES and namespaces and namespaces not in settings.JS_URLS_NAMESPACES:
                        continue
                    if settings.JS_URLS_NAMESPACES_EXCLUDE and namespaces in settings.JS_URLS_NAMESPACES_EXCLUDE:
                        continue
                    new_prefix = '%s%s' % (prefix, pattern.regex.pattern)
                    urls.update(_get_urls(pattern.urlconf_name, new_prefix, namespaces))
    return urls


class LazyJsonEncoder(DjangoJSONEncoder):
    '''
    A JSON encoder handling promises (aka. Django lazy objects).

    See: https://docs.djangoproject.com/en/dev/topics/serialization/#id2
    '''
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyJsonEncoder, self).default(obj)


class ContextSerializer(object):
    '''
    Serialize the context from requests.
    '''
    @classmethod
    def as_dict(cls, request):
        '''
        Serialize the context as a dictionnary from a given request.
        '''
        data = {}
        for context in RequestContext(request):
            for key, value in six.iteritems(context):
                processor_name = 'process_%s' % key
                if hasattr(cls, processor_name):
                    processor = getattr(cls, processor_name)
                    data[key] = processor(value, data)
                elif isinstance(value, (str, tuple, list, dict, int, bool, set)):
                    data[key] = value
        cls.handle_user(data, request)
        return data

    @classmethod
    def as_json(cls, request):
        '''
        Serialize the context as JSON from a given request.
        '''
        return json.dumps(cls.as_dict(request), cls=LazyJsonEncoder)

    @classmethod
    def process_LANGUAGES(cls, languages, data):
        return dict((code, _(name)) for code, name in languages)

    @classmethod
    def process_LANGUAGE_CODE(cls, language_code, data):
        # Dirty hack to fix non included default
        language = translation.get_language_info('en' if language_code == 'en-us' else language_code)
        data['LANGUAGE_NAME'] = language['name']
        data['LANGUAGE_NAME_LOCAL'] = language['name_local']
        return language_code

    @classmethod
    def handle_user(cls, data, request):
        '''Insert user informations in data'''
        # Default to unauthenticated anonymous user
        data['user'] = {
            'username': '',
            'is_authenticated': False,
            'is_staff': False,
            'is_superuser': False,
            'permissions': tuple(),
        }
        if 'django.contrib.sessions.middleware.SessionMiddleware' in settings.MIDDLEWARE_CLASSES:
            data['user'].update({
                'username': request.user.username,
                'is_authenticated': request.user.is_authenticated(),
                'is_staff': request.user.is_staff,
                'is_superuser': request.user.is_superuser,
                'permissions': tuple(request.user.get_all_permissions())
            })


class StorageGlobber(object):
    '''
    Retrieve file list from static file storages.
    '''
    @classmethod
    def glob(cls, files=None):
        '''
        Glob a pattern or a list of pattern static storage relative(s).
        '''
        files = files or []
        if isinstance(files, str):
            matches = lambda path: matches_patterns(path, [files])
            return [path for path in cls.get_static_files() if matches(path)]
        elif isinstance(files, (list, tuple)):
            all_files = cls.get_static_files()
            sorted_result = []
            for pattern in files:
                sorted_result.extend([f for f in all_files if matches_patterns(f, [pattern])])
            return sorted_result

    @classmethod
    def get_static_files(cls):
        files = []
        for finder in finders.get_finders():
            for path, storage in finder.list(None):
                # Prefix the relative path if the source storage contains it
                if getattr(storage, 'prefix', None):
                    prefixed_path = os.path.join(storage.prefix, path)
                else:
                    prefixed_path = path

                if prefixed_path not in files:
                    files.append(prefixed_path)
        return files



# -*- coding: utf-8 -*-
'''
Main access point for all JS commands
'''
from django.core.management.base import BaseCommand
from optparse import make_option

import logging
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Handle javascript operations'

    def handle(self, *args, **options):
        pass



from django.test import TestCase
from django.test.utils import override_settings
from djangojs.urls import js_info_dict


@override_settings(USE_I18N=True, LANGUAGE_CODE='en',
                   INSTALLED_APPS=['djangojs', 'djangojs.fake'],
                   JS_I18N_APPS=['djangojs'])
class I18nIncludeTest(TestCase):
    def setUp(self):
        self.packages = js_info_dict()['packages']

    def test_include_translation(self):
        '''Should include apps listed in JS_I18N'''
        self.assertIn('djangojs', self.packages)

    def test_not_include_translation(self):
        '''Should not include apps not listed in JS_I18N'''
        self.assertNotIn('djangojs.fake', self.packages)


@override_settings(USE_I18N=True, LANGUAGE_CODE='en',
                   INSTALLED_APPS=['djangojs', 'djangojs.fake'],
                   JS_I18N_APPS_EXCLUDE=['djangojs'])
class I18nExcludeTest(TestCase):
    def setUp(self):
        self.packages = js_info_dict()['packages']

    def test_exclude_translation(self):
        '''Should exclude apps listed in JS_I18N_EXCLUDE'''
        self.assertNotIn('djangojs', self.packages)

    def test_not_exclude_translation(self):
        '''Should not exlude apps not listed in JS_I18N_EXCLUDE'''
        self.assertIn('djangojs.fake', self.packages)



# -*- coding: utf-8 -*-
'''
This module provide helper views for javascript.
'''
from __future__ import unicode_literals

import json
import logging
import re

from django.http import HttpResponse
from django.views.generic import View, TemplateView

from djangojs.utils import urls_as_dict, urls_as_json, ContextSerializer, StorageGlobber, LazyJsonEncoder

logger = logging.getLogger(__name__)


__all__ = (
    'JsInitView',
    'JsonView',
    'UrlsJsonView',
    'ContextJsonView',
    'JsTestView',
    'JasmineView',
    'QUnitView',
)

RE_KWARG = re.compile(r"(\(\?P\<(.*?)\>.*?\))")  # Pattern for recongnizing named parameters in urls
RE_ARG = re.compile(r"(\(.*?\))")  # Pattern for recognizing unnamed url parameters
RE_OPT = re.compile(r"\w\?")  # Pattern for recognizing optionnal character
RE_OPT_GRP = re.compile(r"\(\?\:.*\)\?")  # Pattern for recognizing optionnal group

JSON_MIMETYPE = 'application/json'
JAVASCRIPT_MIMETYPE = 'application/javascript'


class JsInitView(TemplateView):
    '''
    Render a javascript file containing the URLs mapping and the context as JSON.
    '''
    template_name = 'djangojs/init.js'

    def get_context_data(self, **kwargs):
        context = super(JsInitView, self).get_context_data(**kwargs)
        context['urls'] = urls_as_json()
        context['context'] = ContextSerializer.as_json(self.request)
        return context

    def render_to_response(self, context, **response_kwargs):
        response_kwargs['content_type'] = JAVASCRIPT_MIMETYPE
        return super(JsInitView, self).render_to_response(context, **response_kwargs)


class JsonView(View):
    '''
    A views that render JSON.
    '''
    def get(self, request, **kwargs):
        data = self.get_context_data(**kwargs)
        return HttpResponse(
            json.dumps(data, cls=LazyJsonEncoder),
            mimetype=JSON_MIMETYPE
        )


class UrlsJsonView(JsonView):
    '''
    Render the URLs as a JSON object.
    '''
    def get_context_data(self, **kwargs):
        return urls_as_dict()


class ContextJsonView(JsonView):
    '''
    Render the context as a JSON object.
    '''
    def get_context_data(self, **kwargs):
        return ContextSerializer.as_dict(self.request)


class JsTestView(TemplateView):
    '''
    Base class for JS tests views
    '''
    #: A path or a list of path to javascript files to include into the view.
    #:
    #: - Supports glob patterns.
    #: - Order is kept for rendering.
    js_files = None
    #: Includes or not Django.js in the test view
    django_js = False
    #: Includes or not jQuery in the test view.
    jquery = False

    def get_context_data(self, **kwargs):
        context = super(JsTestView, self).get_context_data(**kwargs)

        context['js_test_files'] = StorageGlobber.glob(self.js_files)
        context['use_django_js'] = self.django_js
        context['use_query'] = self.jquery

        return context


class JasmineView(JsTestView):
    '''
    Render a Jasmine test runner.
    '''
    template_name = 'djangojs/jasmine-runner.html'


class QUnitView(JsTestView):
    '''
    Render a QUnit test runner
    '''
    template_name = 'djangojs/qunit-runner.html'

    #: QUnit runner theme.
    #:
    #: Should be one of: qunit, gabe, ninja, nv
    theme = 'qunit'

    def get_context_data(self, **kwargs):
        context = super(QUnitView, self).get_context_data(**kwargs)
        context['css_theme'] = 'js/test/libs/%s.css' % self.theme
        return context



# -*- coding: utf-8 -*-
# pylint: disable=W0401
from .test_context import *
from .test_globber import *
from .test_javascript import *
from .test_settings import *
from .test_tags import *
from .test_tap import *
from .test_urls import *
# pylint: enable=W0401


def custom_processor(_):
    '''
    Custom template conext processor for testing purposes.
    '''
    return {'CUSTOM': 'CUSTOM_VALUE'}



# -*- coding: utf-8 -*-
'''
Provide template tags to help with Javascript/Django integration.
'''
from __future__ import unicode_literals

from django import template
from django.contrib.staticfiles.storage import staticfiles_storage
from django.utils import six

from djangojs import JQUERY_MIGRATE_VERSION
from djangojs.conf import settings

register = template.Library()


def verbatim_tags(parser, token, endtagname):
    '''
    Javascript templates (jquery, handlebars.js, mustache.js) use constructs like:

    ::

        {{if condition}} print something{{/if}}

    This, of course, completely screws up Django templates,
    because Django thinks {{ and }} means something.

    The following code preserves {{ }} tokens.

    This version of verbatim template tag allows you to use tags
    like url {% url name %}. {% trans "foo" %} or {% csrf_token %} within.

    Inspired by:
     - Miguel Araujo: https://gist.github.com/893408
    '''
    text_and_nodes = []
    while 1:
        token = parser.tokens.pop(0)
        if token.contents == endtagname:
            break

        if token.token_type == template.TOKEN_VAR:
            text_and_nodes.append('{{')
            text_and_nodes.append(token.contents)

        elif token.token_type == template.TOKEN_TEXT:
            text_and_nodes.append(token.contents)

        elif token.token_type == template.TOKEN_BLOCK:
            try:
                command = token.contents.split()[0]
            except IndexError:
                parser.empty_block_tag(token)

            try:
                compile_func = parser.tags[command]
            except KeyError:
                parser.invalid_block_tag(token, command, None)
            try:
                node = compile_func(parser, token)
            except template.TemplateSyntaxError as e:
                if not parser.compile_function_error(token, e):
                    raise
            text_and_nodes.append(node)

        if token.token_type == template.TOKEN_VAR:
            text_and_nodes.append('}}')

    return text_and_nodes


class VerbatimNode(template.Node):
    '''
    Wrap {% verbatim %} and {% endverbatim %} around a
    block of javascript template and this will try its best
    to output the contents with no changes.

    ::

        {% verbatim %}
            {% trans "Your name is" %} {{first}} {{last}}
        {% endverbatim %}
    '''
    def __init__(self, text_and_nodes):
        self.text_and_nodes = text_and_nodes

    def render(self, context):
        output = ""
        # If its text we concatenate it, otherwise it's a node and we render it
        for bit in self.text_and_nodes:
            if isinstance(bit, basestring):
                output += bit
            else:
                output += bit.render(context)
        return output


@register.tag
def verbatim(parser, token):
    '''Renders verbatim tags'''
    text_and_nodes = verbatim_tags(parser, token, 'endverbatim')
    return VerbatimNode(text_and_nodes)


@register.simple_tag
def js_lib(filename):
    return javascript('js/libs/%s' % filename)


@register.simple_tag
def javascript(filename, type='text/javascript'):
    '''A simple shortcut to render a ``script`` tag to a static javascript file'''
    if '?' in filename and len(filename.split('?')) is 2:
        filename, params = filename.split('?')
        return '<script type="%s" src="%s?%s"></script>' % (type, staticfiles_storage.url(filename), params)
    else:
        return '<script type="%s" src="%s"></script>' % (type, staticfiles_storage.url(filename))


@register.simple_tag
def js(filename, type='text/javascript'):
    '''A simple shortcut to render a ``script`` tag to a static javascript file'''
    return javascript(filename, type=type)


@register.simple_tag
def coffeescript(filename):
    '''A simple shortcut to render a ``script`` tag to a static coffeescript file'''
    return javascript(filename, type='text/coffeescript')


@register.simple_tag
def coffee(filename):
    '''A simple shortcut to render a ``script`` tag to a static coffeescript file'''
    return javascript(filename, type='text/coffeescript')


@register.simple_tag
def css(filename):
    '''A simple shortcut to render a ``link`` tag to a static CSS file'''
    return '<link rel="stylesheet" type="text/css" href="%s" />' % staticfiles_storage.url(filename)


def _boolean(value):
    if isinstance(value, bool):
        return value
    elif isinstance(value, (six.text_type, six.string_types)):
        return value.lower() == 'true'
    elif isinstance(value, int):
        return value != 0
    else:
        return False


@register.simple_tag
def jquery_js(version=None, migrate=False):
    '''A shortcut to render a ``script`` tag for the packaged jQuery'''
    version = version or settings.JQUERY_VERSION
    suffix = '.min' if not settings.DEBUG else ''
    libs = [js_lib('jquery-%s%s.js' % (version, suffix))]
    if _boolean(migrate):
        libs.append(js_lib('jquery-migrate-%s%s.js' % (JQUERY_MIGRATE_VERSION, suffix)))
    return '\n'.join(libs)


@register.inclusion_tag('djangojs/django_js_tag.html', takes_context=True)
def django_js(context, jquery=True, i18n=True, csrf=True):
    '''Include Django.js javascript library in the page'''
    return {
        'js': {
            'minified': not settings.DEBUG,
            'jquery': _boolean(jquery),
            'i18n': _boolean(i18n),
            'csrf': _boolean(csrf),
        }
    }


@register.inclusion_tag('djangojs/django_js_init.html', takes_context=True)
def django_js_init(context, jquery=False, i18n=True, csrf=True):
    '''Include Django.js javascript library initialization in the page'''
    return {
        'js': {
            'jquery': _boolean(jquery),
            'i18n': _boolean(i18n),
            'csrf': _boolean(csrf),
        }
    }



# -*- coding: utf-8 -*-
'''
This module provide test runners for JS in Django.
'''
from __future__ import unicode_literals

import re

from django.utils import termcolors
from django.utils.encoding import python_2_unicode_compatible

green = termcolors.make_style(fg='green', opts=('bold',))
red = termcolors.make_style(fg='red', opts=('bold',))

# TAP regex
TAP_MODULE_REGEX = re.compile(r'^(?P<indent>\s*)# module: (?P<name>.*)$')
TAP_TEST_REGEX = re.compile(r'^(?P<indent>\s*)# test: (?P<name>.*)$')
TAP_ASSERTION_REGEX = re.compile(r'^(?P<indent>\s*)(?P<type>(?:not )?ok) (?P<num>\d+)(?: - (?P<details>.*))?$')
TAP_STACK_REGEX = re.compile(r'^(?P<indent>\s*)#\s+at\s+(?P<stack>.*)$')
TAP_END_REGEX = re.compile(r'^(?P<indent>\s*)(?P<start>\d+)\.\.(?P<end>\d+)(?: - (?P<details>.*))?$')
TAP_DETAILS_REGEX = re.compile(
    r'''^(?:(?P<message>(?!expected)(?!got)(?!matcher)(?!source).+?)(?:, )?)?'''
    r'''(?:expected: '(?P<expected>.+)', got: '(?P<got>.+?)'(?:, )?)?'''
    r'''(?:matcher: '(?P<matcher>.+?)'(?:, )?)?'''
    r'''(?:source:\s+at\s+(?P<source>.+?))?$''')


# Output format
INDENT = 4
DEFAULT_ENCODING = 'utf-8'


class TapItem(object):
    parent = None
    parsed_indent = ''

    @property
    def indent(self):
        if self.parent and isinstance(self.parent, TapItem):
            if isinstance(self.parent, TapModule):
                return self.parent.indent + ' ' * INDENT
            else:
                return self.parent.indent
        return ''


@python_2_unicode_compatible
class TapGroup(list, TapItem):
    def __init__(self, name='', parent=None, parsed_indent='', *args, **kwargs):
        super(TapGroup, self).__init__(*args, **kwargs)
        self.name = name
        self.parent = parent
        self.parsed_indent = parsed_indent

    def __str__(self):
        return '# Groupe %s' % self.name

    def __nonzero__(self):
        return True

    def __bool__(self):
        return True

    def append(self, item):
        if isinstance(item, (TapGroup, TapAssertion)) and not item.parent:
            item.parent = self
        super(TapGroup, self).append(item)

    def get_all_failures(self):
        failures = []
        for item in self:
            if isinstance(item, TapGroup):
                failures.extend(item.get_all_failures())
            elif isinstance(item, TapAssertion) and not item.success:
                failures.append(item)
        return failures


@python_2_unicode_compatible
class TapModule(TapGroup):

    def display(self):
        return '%s%s' % (self.indent, self.name)

    def __str__(self):
        return '# module: %s' % self.name

    @classmethod
    def parse(cls, line):
        match = TAP_MODULE_REGEX.match(line.rstrip())
        if match:
            return cls(
                match.group('name').strip(),
                parsed_indent=match.group('indent')
            )
        else:
            return None


@python_2_unicode_compatible
class TapTest(TapGroup):

    def display(self):
        assertions = [result.display(True) for result in self]
        if assertions:
            return '%s%s (%s)' % (self.indent, self.name, ' '.join(assertions))
        else:
            return '%s%s' % (self.indent, self.name)

    def __str__(self):
        return '# test: %s' % self.name

    @classmethod
    def parse(cls, line):
        match = TAP_TEST_REGEX.match(line.rstrip())
        if match:
            return cls(match.group('name').strip(), parsed_indent=match.group('indent'))
        else:
            return None


@python_2_unicode_compatible
class TapAssertion(TapItem):
    def __init__(self, num, success=True, message=None, parsed_indent='', *args, **kwargs):
        super(TapAssertion, self).__init__()
        self.num = num
        self.success = success
        self.message = message
        self.expected = None
        self.got = None
        self.matcher = None
        self.stack = []
        self.parsed_indent = parsed_indent

    def display(self, inline=False):
        if inline:
            return green('ok') if self.success else red('ko')
        else:
            text = self.indent
            if self.success:
                text += green('ok %s' % self.num)
            else:
                text += red('not ok %s' % self.num)
            text = '%s - %s' % (text, self.message) if self.message else text
            if self.expected is not None and self.got is not None:
                text = '\n'.join([text, '# expected: %s' % self.expected, '# got: %s' % self.got])
            if self.stack:
                text = '\n'.join([text] + ['# stack: %s' % line for line in self.stack])
            return text

    def __str__(self):
        return 'ok %s' % self.num if self.success else 'not ok %s' % self.num

    @classmethod
    def parse(cls, line):
        match = TAP_ASSERTION_REGEX.match(line.rstrip())
        if match:
            assertion = cls(
                int(match.group('num')),
                match.group('type') == 'ok',
                parsed_indent=match.group('indent')
            )
            if match.group('details'):
                details_match = TAP_DETAILS_REGEX.match(match.group('details'))
                if details_match and details_match.group('message'):
                    assertion.message = details_match.group('message')
                if details_match and details_match.group('expected'):
                    assertion.expected = details_match.group('expected')
                    assertion.got = details_match.group('got')
                if details_match and details_match.group('matcher'):
                    assertion.matcher = details_match.group('matcher')
                if details_match and details_match.group('source'):
                    assertion.stack = [details_match.group('source')]
            return assertion
        else:
            return None


HIERARCHY = (
    TapModule,
    TapTest,
    TapAssertion,
)


def hierarchy(item):
    if not isinstance(item, TapItem):
        raise ValueError('Item should be a TapItem')
    return HIERARCHY.index(item.__class__)


class TapParser(object):
    '''
    A TAP parser class reading from iterable TAP lines.
    '''
    def __init__(self, yield_class=TapTest, debug=False):
        if not issubclass(yield_class, TapItem):
            raise ValueError('yield class should extend TapItem')
        self.suites = TapGroup()
        self.current = self.suites
        self.yield_class = yield_class
        self.debug = debug

    def parse(self, lines):
        for line in lines:
            for item in self.parse_line(line):
                yield item
        for item in self.get_lasts():
            yield item

    def parse_line(self, line):
        item = TapModule.parse(line) or TapTest.parse(line) or TapAssertion.parse(line)
        if item is not None:
            return self.set_current(item)

        match = TAP_STACK_REGEX.match(line.rstrip())
        if match:
            self.current.stack.append(match.group('stack'))
            return []

        match = TAP_END_REGEX.match(line.rstrip())
        if match and self.debug:
            print('# end %s-%s' % (match.group('start'), match.group('end')))
            return []

        if line and self.debug:
            print('not matched: %s' % line)

        return []

    def set_current(self, item=None):
        if item and not isinstance(item, TapItem):
            raise ValueError('Should be a TAP item')

        ended = []

        while self.current.parent and hierarchy(item) < hierarchy(self.current):
            if isinstance(self.current, self.yield_class):
                ended.append(self.current)
            self.current = self.current.parent

        while self.current.parent \
                and hierarchy(item) == hierarchy(self.current) \
                and len(item.parsed_indent) <= len(self.current.parsed_indent):
            if isinstance(self.current, self.yield_class):
                ended.append(self.current)
            self.current = self.current.parent

        self.current.append(item)
        self.current = item

        if hierarchy(self.current) < HIERARCHY.index(self.yield_class):
            ended.append(self.current)

        return ended

    def get_lasts(self):
        lasts = []
        while isinstance(self.current, TapItem) and self.current.parent:
            if isinstance(self.current, self.yield_class):
                lasts.append(self.current)
            self.current = self.current.parent
        return lasts



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from setuptools import setup, find_packages


PYPI_RST_FILTERS = (
    # Replace code-blocks
    (r'\.\.\s? code-block::\s*(\w|\+)+',  '::'),
    # Remove travis ci badge
    (r'.*travis-ci\.org/.*', ''),
)


def rst(filename):
    '''
    Load rst file and sanitize it for PyPI.
    Remove unsupported github tags:
     - code-block directive
     - travis ci build badge
    '''
    content = open(filename).read()
    for regex, replacement in PYPI_RST_FILTERS:
        content = re.sub(regex, replacement, content)
    return content


long_description = '\n'.join((
    rst('README.rst'),
    rst('CHANGELOG.rst'),
    ''
))

setup(
    name='django.js',
    version=__import__('djangojs').__version__,
    description=__import__('djangojs').__description__,
    long_description=long_description,
    url='https://github.com/noirbizarre/django.js',
    download_url='http://pypi.python.org/pypi/django.js',
    author='Axel Haustant',
    author_email='noirbizarre+django@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['django'],
    license='LGPL',
    use_2to3=True,
    classifiers=[
        "Framework :: Django",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: System :: Software Distribution",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        "Topic :: Software Development :: Libraries :: Python Modules",
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    ],
)



import unittest

from django.contrib.staticfiles.templatetags.staticfiles import static
from django.core.urlresolvers import reverse
from django.template import Context, Template
from django.test import TestCase
from django.test.utils import override_settings

from djangojs import JQUERY_DEFAULT_VERSION, JQUERY_MIGRATE_VERSION


@unittest.skipIf(__import__('django').VERSION >= (1, 5), "Django 1.5+ already include a verbatim template tag")
class VerbatimTagTest(TestCase):
    def test_rendering(self):
        '''Should escape {{ and }}'''
        template = Template('''
            {% load js %}
            {% verbatim %}
                <p>{{name}}</p>
                {{{rawname}}}
            {% endverbatim %}
            ''')
        rendered = template.render(Context())

        self.assertIn('{{name}}', rendered)
        self.assertIn('{{{rawname}}}', rendered)
        # HTML should not be escaped
        self.assertIn('<p>', rendered)
        self.assertIn('</p>', rendered)

    def test_rendering_with_tags(self):
        '''Should process django template tags'''
        template = Template('''
            {% load i18n js %}

            {% verbatim %}
                {% trans "with translation" %}
                {{name}}
                <p>{{{rawname}}}</p>
                {# works with comments too #}
            {% endverbatim %}
            ''')
        rendered = template.render(Context())

        self.assertIn('{{name}}', rendered)
        self.assertIn('{{{rawname}}}', rendered)
        self.assertIn('with translation', rendered)
        # Those should not be rendered :
        self.assertNotIn('{% trans %}', rendered)
        self.assertNotIn('comments', rendered)
        # HTML should not be escaped
        self.assertIn('<p>', rendered)
        self.assertIn('</p>', rendered)


class DjangoJsTagTest(TestCase):
    urls = 'djangojs.urls'

    def test_js(self):
        '''Should include static js files'''
        template = Template('''
            {% load js %}
            {% js "js/my.js" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s">' % static('js/my.js'), rendered)

    def test_js_custom_type(self):
        '''Should include static js files with custom content type'''
        template = Template('''
            {% load js %}
            {% js "js/my.custom" type="text/custom" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/custom" src="%s">' % static('js/my.custom'), rendered)

    def test_js_url_params(self):
        '''Should include static javascript files with url parameters'''
        template = Template('''
            {% load js %}
            {% js "js/my.js?key=value" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s?%s">' % (static('js/my.js'), 'key=value'), rendered)

    def test_javascript(self):
        '''Should include static javascript files'''
        template = Template('''
            {% load js %}
            {% javascript "js/my.js" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s">' % static('js/my.js'), rendered)

    def test_javascript_custom(self):
        '''Should include static javacript files with custom content type'''
        template = Template('''
            {% load js %}
            {% javascript "js/my.custom" type="text/custom" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/custom" src="%s">' % static('js/my.custom'), rendered)

    def test_coffee(self):
        '''Should include static coffeescript files (short)'''
        template = Template('''
            {% load js %}
            {% coffee "js/my.coffee" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/coffeescript" src="%s">' % static('js/my.coffee'), rendered)

    def test_coffeescript(self):
        '''Should include static coffeescript files'''
        template = Template('''
            {% load js %}
            {% coffeescript "js/my.coffee" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/coffeescript" src="%s">' % static('js/my.coffee'), rendered)

    def test_javascript_url_params(self):
        '''Should include static javascript files with url parameters'''
        template = Template('''
            {% load js %}
            {% javascript "js/my.js?key=value" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s?%s">' % (static('js/my.js'), 'key=value'), rendered)

    def test_css(self):
        '''Should include static css files'''
        template = Template('''
            {% load js %}
            {% css "css/my.css" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<link rel="stylesheet" type="text/css" href="%s" />' % static('css/my.css'), rendered)

    def test_js_lib(self):
        '''Should include js libraries'''
        template = Template('''
            {% load js %}
            {% js_lib "my-lib.js" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s">' % static('js/libs/my-lib.js'), rendered)

    def test_js_lib_url_param(self):
        '''Should include js libraries with url parameters'''
        template = Template('''
            {% load js %}
            {% js_lib "my-lib.js?k=v" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s?%s">' % (static('js/libs/my-lib.js'), 'k=v'), rendered)

    @override_settings(DEBUG=False)
    def test_jquery_js_minified(self):
        '''Should include minified jQuery library when DEBUG=False'''
        template = Template('''
            {% load js %}
            {% jquery_js %}
            ''')
        rendered = template.render(Context())
        jquery = static('js/libs/jquery-%s.min.js' % JQUERY_DEFAULT_VERSION)
        self.assertIn('<script type="text/javascript" src="%s">' % jquery, rendered)

    @override_settings(DEBUG=True)
    def test_jquery_js_unminified(self):
        '''Should include unminified jQuery library when DEBUG=True'''
        template = Template('''
            {% load js %}
            {% jquery_js %}
            ''')
        rendered = template.render(Context())
        jquery = static('js/libs/jquery-%s.js' % JQUERY_DEFAULT_VERSION)
        self.assertIn('<script type="text/javascript" src="%s">' % jquery, rendered)

    def test_jquery_js_version(self):
        '''Should include jQuery library with specified version'''
        template = Template('''
            {% load js %}
            {% jquery_js "1.8.3" %}
            ''')
        rendered = template.render(Context())
        jquery = static('js/libs/jquery-1.8.3')
        self.assertIn('<script type="text/javascript" src="%s' % jquery, rendered)

    @override_settings(DEBUG=False)
    def test_jquery_js_migrate_minified(self):
        '''Should include jQuery minified library with migrate when DEBUG=False'''
        template = Template('''
            {% load js %}
            {% jquery_js migrate="true" %}
            ''')
        rendered = template.render(Context())
        jquery = static('js/libs/jquery-%s.min.js' % JQUERY_DEFAULT_VERSION)
        migrate = static('js/libs/jquery-migrate-%s.min.js' % JQUERY_MIGRATE_VERSION)
        self.assertIn('<script type="text/javascript" src="%s">' % jquery, rendered)
        self.assertIn('<script type="text/javascript" src="%s">' % migrate, rendered)

    @override_settings(DEBUG=True)
    def test_jquery_js_migrate_unminified(self):
        '''Should include jQuery unminified library with migrate when DEBUG=True'''
        template = Template('''
            {% load js %}
            {% jquery_js migrate="true" %}
            ''')
        rendered = template.render(Context())
        jquery = static('js/libs/jquery-%s.js' % JQUERY_DEFAULT_VERSION)
        migrate = static('js/libs/jquery-migrate-%s.js' % JQUERY_MIGRATE_VERSION)
        self.assertIn('<script type="text/javascript" src="%s">' % jquery, rendered)
        self.assertIn('<script type="text/javascript" src="%s">' % migrate, rendered)

    @override_settings(DEBUG=False)
    def test_django_js_minified(self):
        '''Should include and initialize django.js minified when DEBUG=False'''
        template = Template('''
            {% load js %}
            {% django_js %}
            ''')
        rendered = template.render(Context())

        jquery = static('js/libs/jquery-%s.min.js' % JQUERY_DEFAULT_VERSION)
        django_js = static('js/djangojs/django.min.js')
        django_js_init = reverse('django_js_init')
        js_catalog = reverse('js_catalog')

        for script in jquery, django_js, django_js_init, js_catalog:
            self.assertIn('<script type="text/javascript" src="%s">' % script, rendered)

        self.assertIn('window.DJANGO_JS_CSRF', rendered)

    @override_settings(DEBUG=True)
    def test_django_js_unminified(self):
        '''Should include and initialize django.js unminified when DEBUG=True'''
        template = Template('''
            {% load js %}
            {% django_js %}
            ''')
        rendered = template.render(Context())

        jquery = static('js/libs/jquery-%s.js' % JQUERY_DEFAULT_VERSION)
        django_js = static('js/djangojs/django.js')
        django_js_init = reverse('django_js_init')
        js_catalog = reverse('js_catalog')

        for script in jquery, django_js, django_js_init, js_catalog:
            self.assertIn('<script type="text/javascript" src="%s">' % script, rendered)

        self.assertIn('window.DJANGO_JS_CSRF', rendered)

    def test_django_js_jquery_false(self):
        '''Should include django.js without jQuery'''
        template = Template('''
            {% load js %}
            {% django_js jquery="false" %}
            ''')
        jquery = static('js/libs/jquery-%s' % JQUERY_DEFAULT_VERSION)
        django_js = static('js/djangojs/django')

        rendered = template.render(Context())

        self.assertNotIn('<script type="text/javascript" src="%s' % jquery, rendered)
        self.assertIn('<script type="text/javascript" src="%s' % django_js, rendered)

    def test_django_js_csrf_false(self):
        '''Should include django.js without jQuery CSRF patch'''
        template = Template('''
            {% load js %}
            {% django_js csrf="false" %}
            ''')
        rendered = template.render(Context())
        self.assertIn('window.DJANGO_JS_CSRF = false;', rendered)

    def test_django_js_i18n(self):
        '''Should include django.js with i18n support'''
        template = Template('''
            {% load js %}
            {% django_js %}
            ''')
        rendered = template.render(Context())
        self.assertIn('<script type="text/javascript" src="%s">' % reverse('js_catalog'), rendered)

    def test_django_js_i18n_false(self):
        '''Should include django.js without i18n support'''
        template = Template('''
            {% load js %}
            {% django_js i18n="false" %}
            ''')
        rendered = template.render(Context())
        self.assertNotIn('<script type="text/javascript" src="%s">' % reverse('js_catalog'), rendered)

    def test_django_js_init(self):
        '''Should include django.js prerequisites'''
        template = Template('''
            {% load js %}
            {% django_js_init %}
            ''')
        rendered = template.render(Context())

        django_js_init = reverse('django_js_init')
        js_catalog = reverse('js_catalog')

        rendered = template.render(Context())

        for script in django_js_init, js_catalog:
            self.assertIn('<script type="text/javascript" src="%s">' % script, rendered)

        jquery = static('js/libs/jquery-%s' % JQUERY_DEFAULT_VERSION)
        self.assertNotIn('<script type="text/javascript" src="%s' % jquery, rendered)
        self.assertIn('window.DJANGO_JS_CSRF = true;', rendered)

    def test_django_js_init_jquery(self):
        '''Should include django.js prerequisites with jquery'''
        template = Template('''
            {% load js %}
            {% django_js_init jquery="true" %}
            ''')
        rendered = template.render(Context())

        django_js_init = reverse('django_js_init')
        js_catalog = reverse('js_catalog')

        rendered = template.render(Context())

        for script in django_js_init, js_catalog:
            self.assertIn('<script type="text/javascript" src="%s">' % script, rendered)

        jquery = static('js/libs/jquery-%s' % JQUERY_DEFAULT_VERSION)
        self.assertIn('<script type="text/javascript" src="%s' % jquery, rendered)
        self.assertIn('window.DJANGO_JS_CSRF = true;', rendered)

    def test_django_js_init_crsf_false(self):
        '''Should include django.js prerequisites'''
        template = Template('''
            {% load js %}
            {% django_js_init csrf="false" %}
            ''')
        rendered = template.render(Context())

        django_js_init = reverse('django_js_init')
        js_catalog = reverse('js_catalog')

        for script in django_js_init, js_catalog:
            self.assertIn('<script type="text/javascript" src="%s">' % script, rendered)

        self.assertIn('window.DJANGO_JS_CSRF = false;', rendered)

    def test_django_js_init_i18n_false(self):
        '''Should include django.js prerequisites'''
        template = Template('''
            {% load js %}
            {% django_js_init i18n="false" %}
            ''')
        rendered = template.render(Context())

        self.assertIn('window.DJANGO_JS_CSRF', rendered)
        self.assertIn('<script type="text/javascript" src="%s">' % reverse('django_js_init'), rendered)
        self.assertNotIn('<script type="text/javascript" src="%s">' % reverse('js_catalog'), rendered)



# -*- coding: utf-8 -*-
from django import forms
from django.conf.urls import patterns, url, include
from django.core.urlresolvers import reverse
from django.views.generic import TemplateView
from django.views.generic.edit import BaseFormView

from djangojs.views import JasmineView, QUnitView


def unnamed(request, **kwargs):
    return None


class TestForm(forms.Form):
    message = forms.CharField()


class TestFormView(BaseFormView):
    form_class = TestForm

    def get_success_url(self):
        return reverse('opt')


class DjangoJsTestView(JasmineView):
    template_name = 'djangojs/test/djangojs-test-runner.html'
    js_files = 'js/test/django.specs.js'
    django_js = True

    def get_context_data(self, **kwargs):
        context = super(DjangoJsTestView, self).get_context_data(**kwargs)
        context['form'] = TestForm()
        return context


class JasmineTestView(JasmineView):
    js_files = 'js/test/jasmine/*Spec.js'


class QUnitTestView(QUnitView):
    template_name = 'djangojs/test/qunit-test-runner.html'
    js_files = 'js/test/qunit/qunit-*.js'


fake_patterns = patterns('',
    url(r'^fake$', TestFormView.as_view(), name='fake'),
)

nested_patterns = patterns('',
    url(r'^nested/', include(fake_patterns, namespace="nested", app_name="appnested")),
)

other_fake_patterns = patterns('',
    url(r'^fake$', TestFormView.as_view(), name='fake'),
)

test_patterns = patterns('',
    url(r'^form$', TestFormView.as_view(), name='test_form'),
    url(r'^unamed$', 'djangojs.test_urls.unnamed'),
    url(r'^unnamed-class$', TestFormView.as_view()),
    url(r'^arg/(\d+)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_arg'),
    url(r'^arg/(\d+)/(\w)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_arg_multi'),
    url(r'^named/(?P<test>\w+)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_named'),
    url(r'^named/(?P<str>\w+)/(?P<num>\d+)$',
        TemplateView.as_view(template_name='djangojs/test/test1.html'),
        name='test_named_multi'),
    url(r'^optionnals?$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name="opt"),
    url(r'^many?/optionnals?$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name="opt_multi"),
    url(r'^optionnal/(?:capturing)?group$',
        TemplateView.as_view(template_name='djangojs/test/test1.html'),
        name="opt_grp"),
    url(r'^first/$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='twice'),
    url(r'^last/$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='twice'),
    url(r'^namespace1/', include(fake_patterns, namespace="ns1", app_name="app1")),
    url(r'^namespace2/', include(nested_patterns, namespace="ns2", app_name="app2")),
    url(r'^namespace3/', include(fake_patterns, namespace="ns3")),
    url(r'^test\.json$', TestFormView.as_view(), name='escaped'),
)

urlpatterns = patterns('',
    url(r'^$', DjangoJsTestView.as_view(), name='djangojs_tests'),

    url(r'^djangojs/', include('djangojs.urls')),

    url(r'^jasmine/$', JasmineTestView.as_view(), name='djangojs_jasmine_tests'),
    url(r'^qunit/$', QUnitTestView.as_view(), name='djangojs_qunit_tests'),

    url(r'^test/', include(test_patterns)),
)
